//#define BUFFER_DEBUG
using UnityEngine;

public class UAVInPixelShader : MonoBehaviour
{
	public Shader m_Shader;
	public Material m_LineMaterial;
	
	private Material m_Mat = null;
	private Mesh m_Mesh;
	private ComputeBuffer m_CBRed;
	private ComputeBuffer m_CBGreen;
	private ComputeBuffer m_CBBlue;
	
	private const int kCount = 256;
		
	void OnDisable ()
	{
		DestroyImmediate (m_Mat); m_Mat = null;
		DestroyImmediate (m_Mesh); m_Mesh = null;
		if (m_CBRed != null) m_CBRed.Release(); m_CBRed = null;
		if (m_CBGreen != null) m_CBGreen.Release(); m_CBGreen = null;
		if (m_CBBlue != null) m_CBBlue.Release(); m_CBBlue = null;
	}
	
	Mesh CreateMesh ()
	{
		Mesh mesh = new Mesh ();
		Vector3[] vertices = new Vector3[kCount];
		int[] indices = new int[2 * (kCount - 1)];
		Vector3[] normals = new Vector3[kCount];
		Vector2[] uvs = new Vector2[kCount];
		Color32[] colors = new Color32[kCount];
		
		for (int i = 0; i < kCount; ++i)
		{
			vertices[i] = new Vector3 (i/(float)(kCount), i/(float)(kCount), 0);
			if (i < kCount - 1)
			{
				indices[2*i + 0] = i;
				indices[2*i + 1] = i + 1;
			}
			byte rand = (byte)i;
			colors[i] = new Color32 (rand, rand, rand, 255);
		}
		
		mesh.vertices = vertices;
		mesh.normals = normals;
		mesh.uv = uvs;
		mesh.colors32 = colors;
		mesh.subMeshCount = 1;
		mesh.SetIndices (indices, MeshTopology.Lines, 0);
		return mesh;
	}
	
	private static void ClearComputeBuffer (ComputeBuffer cb)
	{
		var values = new int[cb.count];
		for (int i = 0; i < cb.count; i++)
			values[i] = 0;
		cb.SetData (values);
	}
	
	private static ComputeBuffer CreateComputeBuffer ()
	{
		var cb = new ComputeBuffer (256, 4, ComputeBufferType.Default);
		ClearComputeBuffer (cb);
		return cb;		
	}
	
	void CreateResources ()
	{
		if (m_CBRed == null) m_CBRed = CreateComputeBuffer ();
		if (m_CBGreen == null) m_CBGreen = CreateComputeBuffer ();
		if (m_CBBlue == null) m_CBBlue = CreateComputeBuffer ();

		if (m_Mat == null)
		{
			m_Mat = new Material (m_Shader);
			m_Mat.hideFlags = HideFlags.HideAndDontSave;
		}
		
		if (m_Mesh == null)
			m_Mesh = CreateMesh ();
	}
	
	private static void DrawHistogram (ComputeBuffer cb, Color color, Mesh mesh, Material mat)
	{
		mat.SetBuffer ("_HistorgramSB", cb);
		mat.SetPass (0);
		mat.color = color;
		GL.LoadIdentity ();
		Graphics.DrawMeshNow (mesh, Matrix4x4.identity);
	}
	
	void OnRenderImage (RenderTexture src, RenderTexture dst)
	{
		if (!SystemInfo.supportsComputeShaders)
		{
			Graphics.Blit (src, dst);
			Debug.LogWarning ("Compute shaders not supported (not using DX11?)");
			this.enabled = false;
			return;			
		}

		CreateResources ();
		
		// do acumulation of pixel intensity
		Graphics.SetRandomWriteTarget (1, m_CBRed);
		Graphics.SetRandomWriteTarget (2, m_CBGreen);
		Graphics.SetRandomWriteTarget (3, m_CBBlue);
		
		Graphics.Blit (src, dst, m_Mat, 0);
		
		Graphics.ClearRandomWriteTargets();
		Graphics.SetRenderTarget (dst);

#if BUFFER_DEBUG
		var args = new int[256];
		m_CBRed.GetData(args); 
		string bla = "";
		for (int i = 0; i < 256; i++)
			bla += args[i] + " ";
		Debug.Log (Time.realtimeSinceStartup + ": " + bla);
#endif
		DrawHistogram (m_CBRed, Color.red, m_Mesh, m_LineMaterial);
		DrawHistogram (m_CBGreen, Color.green, m_Mesh, m_LineMaterial);
		DrawHistogram (m_CBBlue, Color.blue, m_Mesh, m_LineMaterial);
		
		// tidy up
		ClearComputeBuffer (m_CBRed);
		ClearComputeBuffer (m_CBGreen);
		ClearComputeBuffer (m_CBBlue);
	}
}
