using UnityEngine;
using System.Collections.Generic;

[RequireComponent(typeof(BlockPool))]
public class BlockManager : MonoBehaviour
{
	public Transform m_PlayerTransform;
	public int m_OldBlocksToKeep = 0;
	public bool m_PatchLightProbes = true;
	public float m_LightProbeFixupDistEps = 0.1f;
	public bool m_FixAnchor = true;
	
	private BlockPool m_BlockPool;
	private List<Block> m_Blocks;
	
	public Block m_CurrentBlock;
	private List<Block> m_OldBlocks = new List<Block> ();
	private List<Block> m_NextBlocks = new List<Block> ();
	
	private Transform m_PlayerLightProbeAnchor;
	
	void Start ()
	{
		m_PlayerLightProbeAnchor = m_PlayerTransform.GetComponentInChildren<Renderer> ().lightProbeAnchor;
		
		m_BlockPool = GetComponent<BlockPool> ();
		m_BlockPool.Init ();
		
		// fetch 2 new blocks ahead
		GetNewNextBlock ();
		GetNewNextBlock ();
		
		GetNewCurrentBlock ();
	}
	
	void Update ()
	{
		if (m_PlayerTransform.position.x >= m_CurrentBlock.endPivot.position.x)
		{
			// we just moved to the next block, time to build the path for the player
			Debug.Log ("Moved to the next block.");
			RetireCurrentBlock ();
			GetNewNextBlock ();
			GetNewCurrentBlock ();
		}
		
		ReturnReallyOldBlocksToPool ();
	}
	
	void ReturnReallyOldBlocksToPool ()
	{
		while (m_OldBlocks.Count > 0 && m_OldBlocks.Count > m_OldBlocksToKeep)
		{
			Block reallyOldBlock = m_OldBlocks[0];
			m_OldBlocks.RemoveAt (0);
			m_BlockPool.ReturnBlock (reallyOldBlock);
		}
	}
	
	void RetireCurrentBlock ()
	{
		Debug.Log ("Retiring block.");
		// retire current block
		m_OldBlocks.Add (m_CurrentBlock);
	}
	
	void GetNewLightProbes (int index, Vector3 endPivotLocalOffset, float anchorPositionOffset)
	{
		LightProbes lightProbes = m_BlockPool.GetLightProbesCopyAtIndex (index);
		if (LightmapSettings.lightProbes)
		{
			LightProbes oldLightProbes = LightmapSettings.lightProbes;
			
			if (m_PatchLightProbes)
				LightProbeFixer.PatchLightProbes (oldLightProbes, lightProbes, endPivotLocalOffset, m_LightProbeFixupDistEps);
			
			Destroy (oldLightProbes);
		}
		
		if (m_FixAnchor)
		{
			// fix player's light probe anchor position
			Vector3 anchorLocalPosition = m_PlayerLightProbeAnchor.localPosition;
			anchorLocalPosition.x = -anchorPositionOffset;
			m_PlayerLightProbeAnchor.localPosition = anchorLocalPosition;
		}
		else
		{
			m_PlayerLightProbeAnchor.localPosition = Vector3.zero;
		}
		
		LightmapSettings.lightProbes = lightProbes;
	}
	
	void GetNewCurrentBlock ()
	{
		Vector3 endPivotLocalOffset = Vector3.zero;
		if (m_CurrentBlock)
			endPivotLocalOffset = m_CurrentBlock.endPivot.localPosition;
		
		Debug.Log ("Got new current block.");
		// fetch new current block from the list of next blocks
		m_CurrentBlock = m_NextBlocks[0];
		m_NextBlocks.RemoveAt (0);
		
		float currentBlockPositionOffset = m_CurrentBlock.transform.position.x;
		GetNewLightProbes (m_CurrentBlock.index, endPivotLocalOffset, currentBlockPositionOffset);
	}
	
	void GetNewNextBlock ()
	{
		Debug.Log ("Getting new next block.");
		int nextBlockIndex = m_BlockPool.GetNextBlockIndex ();
		Block nextBlock = m_BlockPool.GetBlockAtIndex (nextBlockIndex);
		Vector3 position = Vector3.zero;
		if (m_NextBlocks.Count > 0)
		{
			Debug.Log ("Placing the new next block after the last next block.");
			Block lastNextBlock = m_NextBlocks[m_NextBlocks.Count - 1];
			position = lastNextBlock.endPivot.position;
		}
		else
		{
			Debug.Log ("No next blocks exist yet, placing ours at (0,0,0).");	
		}
		nextBlock.transform.position = position;
		
		m_NextBlocks.Add (nextBlock);
	}
}
